; Arduino setup() and loop() are defined in this file
.global setup
.global loop

/*------------------------------------------------------------------------
  Include files
------------------------------------------------------------------------*/
#include "stuff.h"

/*------------------------------------------------------------------------
  Arduino setup() function
------------------------------------------------------------------------*/
setup:      cli           ; Disable interrupts

            ; Set CPU to 16MHz clock (for UART)
            ldi pr0,0x80
            ldi pr1,0x01
            sts CLKPR,pr0
            sts CLKPR,pr1

            ; Allow the clock to settle before using the UART
            ldi pr0,0x40
0:          dec pr0
            brne 0b

            ; Initialize UART, print welcome message
            call initUARTd
.stringz
0:         .asciz "Program started OK"
.code
            ldY 0b
            call print_string
            call serial_crlf
            call serial_crlf

            ; Tests of LGT's 16-bit addressing modes
            sbi DSCR,7    ; Enable uDSC
            ;rjmp 6f

            ; IO mode test (pass, with caveats)
            ; Should give "DX: 1234  DY: 5678  DA: 06260060"
1:          ldi pr0,'1'
            rcall TT
            sbi DSCR,6    ; The control bit makes no difference in this mode
            ldi r16,0x34
            ldi r17,0x12
            nop           ; Caveat: Needs a nop here or r17 is lost
            out DSDX,r16
            ldi r17,0x56  ; Swapping the order of r16 and r17 fixes the caveat
            ldi r16,0x78
            out DSDY,r16
            in r8,DSDX    ; Read back DX
            in r10,DSDY   ; Read back DY
            ldi pr0,0x44
            out DSIR,pr0  ; Multiply them, we should get 06260060 in DA
            call dump_cpu

            ; Test 2 - write to registers using RAM address in IO area (fail!)
            ; Should give "DX: 1234  DY: 1234", gives "DX: de34  DY: 1234"
2:          ldi pr0,'2'
            rcall TT
            cbi DSCR,6    ; This setting makes no difference
            ldi r17,0x12
            ldi r16,0x34
            lds r0,db
            lds r1,db+1
            sts DSDX+0x20,r16  ; Fail, r1:r16 is written to DX!
            movw r0,r16        ; r1:r0 works OK (obviously!)
            sts DSDY+0x20,r0
            ldi pr0,0x44
            //out DSIR,pr0
            call dump_cpu

            ; Test 3 - read uDSC registers directly from RAM (fail!)
3:          ldi pr0,'3'
            rcall TT
            cbi DSCR,6        ; uDSC MUST be in MM mode
            lds r0,db+4       ; These shouldn't be overwritten (pass!)
            lds r1,db+5
            ldi r16,0x00
            movw r2,r0
            lds r0,nums+0x2000  ; Value goes into DX (correct)
            lds r1,nums+0x2002  ; Fail! Value goes into DX, not DY
            lds r2,nums+0x2004  ; Fail! Value goes into DX, not DAL
            lds r3,nums+0x2006  ; Fail! Value goes into DX, not DAH
            call dump_cpu

            ; Test 4 - 16 bit indirect load via X register then store to RAM (pass)
            ; Should give "DX: 0123  DY: 4567  DA: 89abcdef" and r16-r23 should show hex digits 0..f
4:          ldi pr0,'4'
            rcall TT
            cbi DSCR,6    ; uDSC MUST be in MM mode
            ldi XL,lo8(nums+0x2000)
            ldi XH,hi8(nums+0x2000)
            ld r0,X+
            sts temp+0x2000,r0 ; Store DX
            lds r16,temp
            lds r17,temp+1
            ld r1,X+
            sts temp+0x2000,r1 ; Store DY
            lds r18,temp
            lds r19,temp+1
            ld r3,X+
            sts temp+0x2000,r3 ; Store DAH
            lds r20,temp
            lds r21,temp+1
            ld r2,X
            sts temp+0x2000,r2 ; Store DAL
            lds r22,temp
            lds r23,temp+1
            call dump_cpu

            ; Test 5 - 16-bit indirect load via Z with offsets (pass!)
            ; Should give "DX: dead  DY: beef  DA: cafebabe"
5:          ldi pr0,'5'
            rcall TT
            cbi DSCR,6    ; uDSC in MM mode
            ldi ZL,lo8(db+0x2000)
            ldi ZH,hi8(db+0x2000)
            ld r0,Z+       ; Value goes in DX
            ldd r1,Z+4     ; Value goes in DY
            ld r2,Z        ; Value goes in DAH
            ld r3,-Z       ; Value goes in DAL
            call dump_cpu

            ; MM/IO mode test - 16-bit indirect load with register
            ; I can't get this mode to do anything at all...
6:          ldi pr0,'6'
            rcall TT
            sbi DSCR,6      // MM mode
            ldi ZL,lo8(nums+0x2000)
            ldi ZH,hi8(nums+0x2000)
            ldi r16,0x11
            mov r0,r16
            ldd r0,Z+0
            call dump_cpu

            jmp clock_test

.stringz
0:         .asciz "Logic Green uDSC test #"
.code
TT:         ldY 0b
            call print_string
            call serial_send
            call serial_crlf
            cbi DSCR,6    ; uDSC in I/O mode
            ldi r17,0x00
            ldi r16,0x00
            out DSDX,r16
            out DSDY,r16
            out DSAL,r16
            out DSAH,r16
            ret


; Output a signal on the DAC (pin 4 on the LGT board)
            cli
            sbi DDRB,0
            ldi pr0,0x0c
            sts DACON,pr0
            ldi pr2,0xff
0:          ldi pr0,0x20
1:          dec pr0
            brne 1b
2:          sts DALR,pr1
            eor pr1,pr2
            //addi pr1,1
            rjmp 0b

/*------------------------------------------------------------------------
  Arduino loop() function
------------------------------------------------------------------------*/
loop:       ret     ; It doesn't do much

/*------------------------------------------------------------------------
  Other functions
------------------------------------------------------------------------*/
; Output a square wave on pin 8 so I can see the system clock frequency on the Micsig...
clock_test:
            ; Set a clock source/frequency
            ldi pr0,0x80
            ldi pr1,0x01
            sts CLKPR,pr0
            sts CLKPR,pr1
         
            sbi DDRB,0  ; Set pin 8 as output
0:          sbi PINB,0  ; Flip the pin as fast as possible...
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            sbi PINB,0
            rjmp 0b       ; There'll be a small glitch here

; This prevents the linker from bombing on odd-sized .S files...
.balign 2
/**/
