/*------------------------------------------------------------------------
  A bunch of definitions for the ASM
------------------------------------------------------------------------*/
#define __SFR_OFFSET 0
#include <avr/io.h>

#define LGT8

#ifdef LGT8

; uDSC
#define DSCR 0x00
#define DSIR 0x01
#define DSDX 0x10
#define DSDY 0x11
#define DSAL 0x38
#define DSAH 0x39

; DAC
#define DACON 0xa0
#define DALR 0xa1

#endif

/*------------------------------------------------------------------------
  Programming defs
------------------------------------------------------------------------*/
; These select linker subsections inside the program
#define code text 0
#define stringz text 1

; Define some registers that can be used with any AVR instruction
#define pr0 r24
#define pr1 r25
#define pr2 r23

/*------------------------------------------------------------------------
  Useful macros
------------------------------------------------------------------------*/
.macro ldY label
  ldi YL,lo8(\label)
  ldi YH,hi8(\label)
.endm

.macro  addi r,val
  subi  \r,-(\val)
.endm

/*------------------------------------------------------------------------
 Set up the UART for debug output
------------------------------------------------------------------------*/
initUARTd:  push pr0
            ldi pr0,0x42      ; Double speed
            sts UCSR0A,pr0
            ldi pr0,0x08      ; Enable transmitter
            sts UCSR0B,pr0
            ldi pr0,0x06      ; 8N1
            sts UCSR0C,pr0
            ldi pr0,0x00
            sts UBRR0H,pr0
            ;ldi pr0,0x10      ; 115200 baud
            ldi pr0,0xCF      ; 9600 baud
            sts UBRR0L,pr0
            pop pr0
            ret

; Send the byte in pr0 to the serial port
serial_send:push pr0
0:          lds pr0,UCSR0A    ; Wait for previous byte to be sent
            sbrs pr0,5
            rjmp 0b
            pop pr0
            sts UDR0,pr0      ; Send the new byte
            ret

/*--------------------------------------------------------------------
  Useful functions for serial output
--------------------------------------------------------------------*/
.global serial_crlf
serial_crlf:push pr0
            ldi pr0,0x0a
            call serial_send
            pop pr0
            ret
.global serial_space
serial_space:push pr0
            ldi pr0,' '
            call serial_send
            pop pr0
            ret

/*--------------------------------------------------------------------
  Print numbers in various ways
--------------------------------------------------------------------*/
; Print contents of pr0 in decimal
print_pr0u: push pr0
            push pr1
            mov pr1,pr0
            ldi pr0,'0'
0:          cpi pr1,100
            brcs 1f
            inc pr0
            subi pr1,100
            rjmp 0b
1:          cpi pr0,'0'
            breq 2f
            call serial_send
2:          ldi pr0,'0'
3:          cpi pr1,10
            brcs 4f
            inc pr0
            subi pr1,10
            rjmp 3b
4:          cpi pr0,'0'
            breq 5f
            call serial_send
5:          mov pr0,pr1
            addi pr0,'0'
            call serial_send
            pop pr1
            pop pr0
            ret

; Print r1:r0 as hexadecimal
print_r0r1h:push r0
            mov r0,r1
            rcall print_r0h
            pop r0
print_r0h:  swap r0
            call 0f
            swap r0
            call 0f
            ret
0:          push pr0
            mov pr0,r0
            andi pr0,0x0f
            cpi pr0,10
            brlo 1f
            addi pr0,('a'-'9'-1)
1:          addi pr0,'0'
            call serial_send
            pop pr0
            ret

/*--------------------------------------------------------------------
  Send zero terminated string (pointed to by Y) from flash/SRAM
--------------------------------------------------------------------*/
print_string:push ZL
            push ZH
            push pr0
0:          movw Z,Y
            lpm pr0,Z+
            movw Y,Z
            and pr0,pr0
            breq 1f
            call serial_send
            rjmp 0b
1:          pop pr0
            pop ZH
            pop ZL
            ret
print_string_s:
            push pr0
            push YH
            push YL
            mov pr0,YL
            or pr0,YH
            breq 1f
0:          ld pr0,Y+
            and pr0,pr0
            breq 1f
            call serial_send
            rjmp 0b
1:          pop YL
            pop YH
            pop pr0
            ret
/*--------------------------------------------------------------------
  Dump all CPU state
--------------------------------------------------------------------*/
dump_cpu:   push r0
            push r1
            push YL
            push YH
            push pr1
            ldi pr1,0
            rcall 0f
            ldi pr1,8
            movw r0,r8
            rcall 0f
            ldi pr1,16
            movw r0,r16
            rcall 0f
            pop pr1
            movw r0,r24
            push pr1
            ldi pr1,24
            rcall 0f
            rcall serial_crlf

            ldi pr1,2
            movw r0,r2
            rcall 0f
            ldi pr1,10
            movw r0,r10
            rcall 0f
            ldi pr1,18
            movw r0,r18
            rcall 0f
            ldY  10f
            rcall print_string
            rcall serial_space
            movw r0,r26
            rcall print_r0r1h
            rcall serial_crlf

            ldi pr1,4
            movw r0,r4
            rcall 0f
            ldi pr1,12
            movw r0,r12
            rcall 0f
            ldi pr1,20
            movw r0,r20
            rcall 0f
            ldY  11f
            rcall print_string
            rcall serial_space
            pop pr1
            pop YH
            pop YL
            movw r0,r28
            push YL
            push YH
            push pr1
            rcall print_r0r1h
            rcall serial_crlf

            ldi pr1,6
            movw r0,r6
            rcall 0f
            ldi pr1,14
            movw r0,r14
            rcall 0f
            ldi pr1,22
            movw r0,r22
            rcall 0f
            ldY  12f
            rcall print_string
            rcall serial_space
            movw r0,r30
            rcall print_r0r1h
            rcall serial_crlf

#ifdef LGT8
            sbis DSCR,7   ;If uDSC is disabled
            rjmp 8f       ; Skip this
            in pr1,DSCR   ; Save DSCR state
            sbi DSCR,6    ; uDSC in IO mode
            ldY  20f
            rcall print_string
            in r0,DSDX
            rcall print_r0r1h
            ldY  21f
            rcall print_string
            in r0,DSDY
            rcall print_r0r1h
            ldY  22f
            rcall print_string
            in r0,DSAH
            rcall print_r0r1h
            in r0,DSAL
            rcall print_r0r1h
            out DSCR,pr1  ; Restore state
            rcall serial_crlf
#endif
8:          rcall serial_crlf
            pop pr1
            pop YH
            pop YL
            pop r1
            pop r0
            ret
0:          push pr0
            push pr1
            inc pr1
            rcall 2f
            ldi pr0,':'
            rcall serial_send
            pop pr1
            rcall 2f
            pop pr0
            rcall serial_space
            cpi pr1,8
            brne 1f
            rcall serial_space
            rcall serial_space
1:          rcall print_r0r1h
            rcall serial_space
            rcall serial_space
            ret
2:          ldi pr0,'r'
            rcall serial_send
            mov pr0,pr1
            rjmp print_pr0u
.stringz
10:         .asciz "     X:"
11:         .asciz "     Y:"
12:         .asciz "     Z:"
#ifdef LGT8
20:         .asciz "DX: "
21:         .asciz "  DY: "
22:         .asciz "  DA: "
#endif
.code

/**/
